// Demo.cpp : demonstrate how to use the library of raid-subsystem
//
#include <algorithm>
#include <vector>
#include <iostream>

// the most important header file for RAID controllers
#include "arclib.h"	

// header for SAS HBAs
#include "arclib-sas.h"

#include "ethernet.h"


#ifdef _DEBUG
	#define ARCDLL_PATH "..\\..\\Debug\\ArcMidDll.dll"
#else
	#define ARCDLL_PATH "..\\..\\Release\\ArcMidDll.dll"
#endif

CArclib *ar;

typedef int (*DLLFINDCONTROLLER)(void);
typedef CArclib * (*DLLGETCONTROLLER)(int idx);

DLLFINDCONTROLLER pDllFindController;
DLLGETCONTROLLER pDllGetController;


using namespace std;

ARC_STATUS Exe_Hw_Info( CArclib &ctrl )
{
	ARC_STATUS  status;
	sSYSTEM_INFO    sysInfo;
	sHwMonInfo  voltageInfo, fanInfo, hddTempInfo, batteryInfo;
	sHwMon5010  hwInfo5010;
	int     i;
	int		addTemp;
	bool    got5010 = false;
	bool    gotRaidCard = false;

	char strId[64];
	char modelStr[9];

	memset(strId, 0, sizeof(strId));
	memset(modelStr, 0, sizeof(modelStr));

	status = ctrl.ArcGetIdentifyString(strId);
	if ( status != ARC_SUCCESS )
	{
		return status;
	}

	memcpy(modelStr, (const void*)&strId[20], 8);


	if ( strcmp(modelStr, "ARC-5010")==0 || strcmp(modelStr, "ARC-6010")==0 )
	{
		got5010 = true;
	}

	if ( strncmp(modelStr, "ARC-11", 6) == 0 || strncmp(modelStr, "ARC-12", 6)==0 )
	{
		gotRaidCard = true;
	}

	if ( got5010 )
	{
		status = ctrl.ArcGetHWMon5010(&hwInfo5010);
		if ( status != ARC_SUCCESS )
		{
			return status;
		}
	}
	else
	{
		status = ctrl.ArcGetSysInfo(&sysInfo);
		if ( status != ARC_SUCCESS )
		{
			return status;
		}

		status = ctrl.ArcGetHWMon(HWMON_FAN_INFO, &fanInfo);
		if ( status != ARC_SUCCESS )
		{
			return status;
		}

		status = ctrl.ArcGetHWMon(HWMON_TMP_INFO, &hddTempInfo);
		if ( status != ARC_SUCCESS )
		{
			return status;
		}

		status = ctrl.ArcGetHWMon(HWMON_VOL_INFO, &voltageInfo);
		if ( status != ARC_SUCCESS )
		{
			return status;
		}

		status = ctrl.ArcGetHWMon(HWMON_BATTERY_INFO, &batteryInfo);
		if ( status != ARC_SUCCESS )
		{
			return status;
		}

	}

	printf("The Hardware Monitor Information\n");
	printf("===========================================\n");
	if ( got5010 )	  // 5 bays ARC-5010 or ARC-6010
	{
		printf("Controller Temp : %d\n", hwInfo5010.monCtrlTemp);
		printf("BackPlane Temp  : %d\n", hwInfo5010.monBkPlaneTemp);
		printf("Fan Speed (RPM) : %d\n", hwInfo5010.monFan);
		printf("Power +12.00V   : %2.3lf\n", (double)hwInfo5010.mon12V/(double)1000.0);
		printf("Power +5.00V    : %2.3lf\n", (double)hwInfo5010.mon5V/(double)1000.0);
		printf("Power +3.30V    : %2.3lf\n", (double)hwInfo5010.mon3_3V/(double)1000.0);
		printf("Power +1.50V    : %2.3lf\n", (double)hwInfo5010.mon1_5V/(double)1000.0);
	}
	else if ( gotRaidCard )	   // RaidCard
	{
		for ( i = 0; i < fanInfo.HwItemCount; i++ )
		{
			if ( fanInfo.ItemInfo[i] == 0xffff )
			{
				printf("Fan#%d Speed (RPM)  : N.A.\n", i+1);
			}
			else
			{
				printf("Fan#%d Speed (RPM)  : %d\n", i+1, fanInfo.ItemInfo[i]);
			}
		}

		for ( i = 0; i < batteryInfo.HwItemCount; i++ )
		{
			if ( batteryInfo.ItemInfo[i] == BBM_NOT_INSTALLED )
			{
				printf("Battery Status      : Not Installed\n");
			}
			else if ( batteryInfo.ItemInfo[i] == BBM_FAILED )
			{
				printf("Battery Status      : Failed\n");
			}
			else
			{
				printf("Battery Status      : %d%%\n", batteryInfo.ItemInfo[i]);
			}
		}

		// show temperatures 
		addTemp = 0;
		i = 0;

		if(hddTempInfo.HwItemCount > sysInfo.gsiIdeChannels)
		{
			printf("CPU Temp.           : %d\n", hddTempInfo.ItemInfo[0]);
			printf("Controller Temp.    : %d\n", hddTempInfo.ItemInfo[1]);
			addTemp = 2;
		}

		for (i = 0; addTemp < hddTempInfo.HwItemCount; i++, addTemp++)
		{
			printf("HDD #%-2d Temp.       : %d\n", i + 1, hddTempInfo.ItemInfo[addTemp]);
		}            
	}
	else // subsystem
	{
		for ( i = 0; i < voltageInfo.HwItemCount; i+=2 )
		{
			printf("Power +%-5.2lfV       : %2.3lf\n", (double)voltageInfo.ItemInfo[i]/(double)1000.0, 
				   (double)voltageInfo.ItemInfo[i+1]/(double)1000.0);
		}

		for ( i = 0; i < fanInfo.HwItemCount; i++ )
		{
			if ( fanInfo.ItemInfo[i] == 0xffff )
			{
				printf("Fan#%d Speed (RPM)  : N.A.\n", i+1);
			}
			else
			{
				printf("Fan#%d Speed (RPM)  : %d\n", i+1, fanInfo.ItemInfo[i]);
			}
		}

		// show controller temperature first
		printf("Controller Temp.    : %d\n", hddTempInfo.ItemInfo[0]);
		//printf("Battery Status     : Not Installed\n");
		for ( i = 1; i < hddTempInfo.HwItemCount; i++ )
		{
			printf("HDD #%-2d Temp.       : %d\n", i, hddTempInfo.ItemInfo[i]);
		}       

	}
	printf("===========================================\n");

	return ARC_SUCCESS;
}

ARC_STATUS Exe_SAS_Hw_Info( CArclib &ctrl )
{
	ARC_STATUS  status = ARC_SUCCESS;   
	BYTE max_enc = 0;
	sSYSTEM_INFO    sysInfo;
	memset(&sysInfo, 0, sizeof(sSYSTEM_INFO));
	status = ctrl.ArcGetSysInfo(&sysInfo);
	if ( status != ARC_SUCCESS )
	{
		return status;
	}
	max_enc = sysInfo.gsiNumberEnclosures;
	//
	// The controller itself h/w monitor information resides at (max enclosure index + 1)
	// ex: A raidset has max up to 8 enclosures, the enclosure index is from 0~7,
	//     then the controller h/w monitor information is index 8
	BYTE contrller_index = max_enc;    
	sSENSORS sensors;
	int loop;

	printf("The Hardware Monitor Information\n");
	printf("=====================================================\n");
	printf("[Controller H/W Monitor]\n");
	status = ctrl.ArcGetEnclosureHwMonitor(contrller_index, &sensors);
	if ( status == ARC_SUCCESS )
	{
		for (loop = 0; loop < sensors.temperature.snNumberOfElements; loop++)
		{
			printf("  %-16s : %ld C\n", 
				   sensors.temperature.snElementStr[loop], 
				   sensors.temperature.snValue[loop]);
		}

		for (loop = 0; loop < sensors.fan.snNumberOfElements; loop++)
		{
			printf("  %-16s : %ld RPM\n", 
				   sensors.fan.snElementStr[loop], 
				   sensors.fan.snValue[loop]);
		}

		for (loop = 0; loop < sensors.voltage.snNumberOfElements; loop++)
		{
			printf("  %-16s : %2.3lf V\n", 
				   sensors.voltage.snElementStr[loop], 
				   (double)(sensors.voltage.snValue[loop]) / 1000);
		}

		for (loop = 0; loop < sensors.battery.snNumberOfElements; loop++)
		{
			if ( sensors.battery.snValue[loop] == BBM_NOT_INSTALLED )
			{
				printf("  %-16s : Not Installed\n", sensors.battery.snElementStr[loop]);
			}
			else if ( sensors.battery.snValue[loop] == BBM_FAILED )
			{
				printf("  %-16s : Failed\n", sensors.battery.snElementStr[loop]);
			}
			else
			{
				printf("  %-16s : %d%%\n", sensors.battery.snElementStr[loop], sensors.battery.snValue[loop]);
			}
		}
	}

	sGUI_ENC_PROPERTY encpro;
	BYTE    prodname[17];
	BYTE    vendorname[9];
	BYTE    rev[5];

	for ( int index = 0; index < sysInfo.gsiNumberEnclosures; index++ )
	{
		status = ctrl.ArcGetEnclosureProperty(index, &encpro);
		if ( status != ARC_SUCCESS || encpro.gepEncType == ENC_TYPE_NOT_INSTALLED )
			continue;


		// enclosure description
		memset(prodname, 0, sizeof(prodname));
		memset(vendorname, 0, sizeof(vendorname));
		memset(rev, 0, sizeof(rev));
		memcpy(prodname, encpro.gepProduct, sizeof(encpro.gepProduct));
		memcpy(vendorname, encpro.gepVendor, sizeof(encpro.gepVendor));
		memcpy(rev, encpro.gepRev, sizeof(encpro.gepRev));

		printf("[Enclosure#%d : %s%s%s]\n", index+1, vendorname, prodname, rev);

		status = ctrl.ArcGetEnclosureHwMonitor(index, &sensors);
		if ( status == ARC_SUCCESS )
		{
			for (loop = 0; loop < sensors.voltage.snNumberOfElements; loop++)
			{
				printf("  %-16s : %2.3lf V\n", 
					   sensors.voltage.snElementStr[loop], 
					   (double)(sensors.voltage.snValue[loop]) / 1000);
			}
			for (loop = 0; loop < sensors.fan.snNumberOfElements; loop++)
			{
				printf("  %-16s : %ld RPM\n", 
					   sensors.fan.snElementStr[loop], 
					   sensors.fan.snValue[loop]);
			}

			for (loop = 0; loop < sensors.power.snNumberOfElements; loop++)
			{
				printf("  %-16s : %s\n", 
					   sensors.power.snElementStr[loop], 
					   sensors.power.snValue[loop] == 1 ? "OK" : "FAILED");
			}

			for (loop = 0; loop < sensors.temperature.snNumberOfElements; loop++)
			{
				if ( sensors.temperature.snValue[loop] == -20 )
				{
					printf("  %-16s : %s\n", 
						   sensors.temperature.snElementStr[loop], "N.A.");
				}
				else
				{
					printf("  %-16s : %ld C\n", 
						   sensors.temperature.snElementStr[loop], 
						   sensors.temperature.snValue[loop]);
				}
			}
		}
	}


	printf("=====================================================\n");  
	return status;
}

#define		UNKNOWN_EVT_STR	"Unknown Event"
/*****************************************************************
Function: Show the event(s) of raid-subsystem
Input:
Output:	1. ARC_STATUS
*****************************************************************/
ARC_STATUS Exe_Event_Info( CArclib &ctrl )
{
	ARC_STATUS  status = ARC_SUCCESS;
	char        ptr1[64];				// temp space of string		
	char        tmpStr[64];
	int         nCount = 0;				// total number of event objects we got	
	int         i;  
	char        timeBuf[64];
	int         nEventFlag = FLAG_ALL;	// the flag pass into ArcGetReqEventPage(...)
	int         nTmpEventType;
	sSYSTEM_INFO    sysInfo;

	//the time event occurred, defined in event.h
	pSYS_TIME evtTime = (pSYS_TIME)new sSYS_TIME;
	// event object defined in event.h
	pEVENT_DATA pEventObject = (pEVENT_DATA)new sEVENT_DATA;    

	*((DWORD *)evtTime) = 0L;
	memset(pEventObject, 0, sizeof(sEVENT_DATA));

	status = ctrl.ArcGetSysInfo(&sysInfo);
	if ( status != ARC_SUCCESS )
	{
		goto quit;
	}

	/* 
		We should call ArcSetTime() to make the system time correct
	*/
	status = ctrl.ArcSetTime();
	if ( status != ARC_SUCCESS )
	{
		goto quit;
	}
	/* 
		Call ArcGetReqEventPage function to get the required event page from raid subsystem.
		There are 4 event pages reside in raid subsystem and each contains 16 event objects.			
	*/
	status = ctrl.ArcGetReqEventPage(nEventFlag, &nCount);
	if ( status != ARC_SUCCESS )
	{
		goto quit;
	}
	if ( status == ARC_SUCCESS )
	{
		if ( nCount != 0 )
		{
			printf("Date-Time               Device              Event Type	       \n");
			printf("===============================================================================\n");
			for ( i = 0; i <= nCount-1; i++ )
			{
				/*
					Call ArcGetEventObject function to extract the individual event object from event page
				*/
				if ( ctrl.ArcGetEventObject( i, pEventObject ) )
				{
					memset(timeBuf, 0, sizeof(timeBuf));
					// If tmFlag == 1 ==> use YY/MM/DD/HH/MM/SS else use time tick
					*((DWORD *)evtTime) = pEventObject->evtTime;
					if ( *((DWORD *)evtTime) & 0x80000000 )
					{
						sprintf(timeBuf, "%04d-%02d-%02d  %02d:%02d:%02d", 
								evtTime->u.tmYear + 2000, 
								evtTime->u.tmMonth,
								evtTime->u.tmDate,
								evtTime->u.tmHour,
								evtTime->u.tmMinute,
								evtTime->u.tmSecond);
					}
					else
					{
						sprintf(timeBuf, "%d", evtTime->x.tmTick );
					}

					memset( ptr1, 0, sizeof(ptr1));
					memset( tmpStr, 0, sizeof(tmpStr));

					/* 
						Parse the event object we got
					*/
					switch ( (int)pEventObject->evtCategory )
					{
					/*
						The member "evtStr" of event object structure contains the string to indicate
						which raidset owns this event object
					*/
					case EVENT_RAIDSET:		// raidset event		
						nTmpEventType = pEventObject->evtType;
						memcpy(tmpStr, pEventObject->evtStr, 16);
						printf("%-24s", timeBuf);
						if ( nTmpEventType > RS_EVT_NO_EVENT )
						{
							printf("%-20s%s\n", tmpStr, UNKNOWN_EVT_STR);
						}
						else
						{
							printf("%-20s%s\n", tmpStr, htmRaidEvent[pEventObject->evtType]);
						}                                           
						break;

						/*
							The member "evtStr" of event object structure contains the string to indicate
							which volumeset owns this event object
						*/
					case EVENT_VOLUMESET: // volumeset event
						nTmpEventType = pEventObject->evtType;
						memcpy(tmpStr, pEventObject->evtStr, 16);
						printf("%-24s", timeBuf);
						if ( nTmpEventType >= VS_EVT_TOTALS )
						{
							printf("%-20s%s\n", tmpStr, UNKNOWN_EVT_STR);
						}
						else
						{
							printf("%-20s%s\n", tmpStr, htmVolEvent[pEventObject->evtType]);
						}                   
						break;

					case EVENT_DEVICE:	// device event
						nTmpEventType = pEventObject->evtType;
						if ( sysInfo.gsiTargetType == TARGET_SAS )
						{
							memcpy(tmpStr, pEventObject->evtStr, 16);
						}
						else
						{
							sprintf(tmpStr, "IDE Channel #%02d", pEventObject->evtChannel + 1);  
						}
						printf("%-24s", timeBuf);
						if ( nTmpEventType >= DEV_EVT_TOTALS )
						{
							printf("%-20s%s\n", tmpStr, UNKNOWN_EVT_STR);
						}
						else
						{
							printf("%-20s%s\n", tmpStr, htmDevEvent[pEventObject->evtType]);
						}                                                                       
						break;

					case EVENT_HOST:	// host event
						/* this is Host SCSI events */
						nTmpEventType = pEventObject->evtType;
						printf("%-24s", timeBuf);
						if ( sysInfo.gsiScsiHostChannels )
						{
							if ( nTmpEventType >= SCSI_EVT_TOTALS )
							{
								printf("%-20s%s\n",
									   eventCat[pEventObject->evtCategory],
									   UNKNOWN_EVT_STR);
							}
							else
							{
								printf("%-20s%s\n",
									   eventCat[pEventObject->evtCategory],
									   htmScsiHostEvent[pEventObject->evtType]);
							}
						}
						else
						{
							if ( nTmpEventType >= SCSI_EVT_TOTALS )
							{
								printf("%-20s%s\n",
									   eventCat[pEventObject->evtCategory],
									   UNKNOWN_EVT_STR);
							}
							else
							{
								printf("%-20s%s\n",
									   eventCat[pEventObject->evtCategory],
									   htmIdeHostEvent[pEventObject->evtType]);
							}
						}
						break;

					case EVENT_HW_MONITOR:	// hardware monitor event
						nTmpEventType = pEventObject->evtType;
						printf("%-24s", timeBuf);
						if ( nTmpEventType >= HW_EVT_TOTALS )
						{
							printf("%-20s%s\n",
								   eventCat[pEventObject->evtCategory],
								   UNKNOWN_EVT_STR);
						}
						else
						{
							printf("%-20s%s\n",
								   eventCat[pEventObject->evtCategory],
								   htmHwMonEvent[pEventObject->evtType]);
						}
						break;
					case EVENT_NEW_83782D:
						memcpy(tmpStr, pEventObject->evtStr, 16);
						printf("%-24s", timeBuf);
						if ( pEventObject->evtType >= GHM_TOTALS )
						{
							printf("%-20s%s\n", tmpStr, UNKNOWN_EVT_STR);
						}
						else
						{
							printf("%-20s%s\n", tmpStr, htmNewEventStr[pEventObject->evtType]);
						}
						break;
					case EVENT_NO_EVENT:
//							printf("		No Event\n");
					default:
						break;

					}// end switch
				}// end if
			}// end for
			printf("===============================================================================\n");
		}//end if
	}// end if

	quit:
	if ( pEventObject )
		delete pEventObject;
	if ( evtTime )
		delete evtTime;

	return status;
}

ARC_STATUS  demo_create_raidset()
{
	ARC_STATUS      status;
	sDRV_MAP        drvMap;			// the buffer to contain the devices that were found by the raid controller
	sDEVICE_LIST    devList;		// the device list to create raidset	
	DWORD           nDevMask = 0;	// devices index for creating raidset(Non-SAS)
	sGUI_PHY_DRV    *drvInfo;		// see arclib.h for details		
	sSYSTEM_INFO    sysInfo;		// see arclib.h for details		
	vector <int>    v1;				// devices index for creating raidset(SAS)

	memset(&sysInfo, 0, sizeof(sysInfo));

	// use the controller#0 for demonstrating
	status = ar->ArcGetSysInfo(&sysInfo);
	if ( status != ARC_SUCCESS )
	{
		printf("Failed to ArcGetSysInfo(...), Error Code = %X.\n", status);
		return status;
	}

	// check password
	status = ar->ArcChkPassword("0000");
	if ( status != ARC_SUCCESS )
	{
		printf("Failed to ArcChkPassword(...), Error Code = %X.\n", status);
		return status;
	}

	// device enumeration
	status = ar->ArcHelpBuildDeviceMap(&drvMap);
	if ( status != ARC_SUCCESS )
	{
		printf("Failed to ArcHelpBuildDeviceMap(...), Error Code = %X.\n", status);
		return status;
	}

	// For demonstrating we suppose creating raidset with device 0~3, 
	// modify this with the real configuration
	v1.push_back(0);
	v1.push_back(1);
	v1.push_back(2);
	v1.push_back(3);

	// check to see if drive is available for creating raidset
	for ( int index = 0; index < (int)v1.size(); index++ )
	{
		drvInfo = &drvMap.drv[v1[index]].drvInfo;
		if ( !(drvInfo->gpdDeviceState & DEV_AVAILABLE) )
		{
			printf("Drive %d Is Not Available\n", v1[index]);
			return ARC_FAILURE;
		}
		else if ( drvInfo->gpdRaidNumber != 0xff )
		{
			printf("Drive %d Is Not Free\n", v1[index]);
			return ARC_FAILURE;                     
		}
		else
		{
			// fill out the device list for creating raidset
			// if Non-SAS the devList.device_pair[index].nEncIndex is always set to 0
			devList.device_pair[index].nEncIndex = drvMap.drv[v1[index]].encindex;
			devList.device_pair[index].nDrvIndex = drvMap.drv[v1[index]].drvindex;          
		}
	}

	// create raidset with default raidset name
	devList.total = (BYTE)v1.size();
	status = ar->ArcCreateRaidSet((pDEVICE_LIST)&devList, NULL);
	if ( status != ARC_SUCCESS )
	{
		printf("Failed to ArcCreateRaidSet(...), Error Code = %X.\n", status);
		return status;
	}

	
	sVolumeSetCap   volCap;
	memset(&volCap, 0, sizeof(sVolumeSetCap));
	volCap.structs_size = sizeof(sVolumeSetCap);
	volCap.raid = 0;	 // create this volume inside the first RaidSet
	strcpy((char *)volCap.volname, "VolumeSet#1");
	volCap.capacity = 10.0;
	volCap.raidlevel = VOLUME_TYPE_RAID5;
	volCap.stripsize = STRIPE_SIZE_64K;
	volCap.channel = 0;
	volCap.id = 0;
	volCap.lun = 0;
	volCap.tag = ENABLE;
	volCap.cache = ENABLE;
	volCap.speed = SCSI_SPEED_160;
	volCap.quickinit = FOREGROUND_INIT;

	status = ar->ArcCreateVolSet(&volCap);
	if ( status != ARC_SUCCESS )
	{
		printf("Failed to ArcCreateVolSet(...), Error Code = %X.\n", status);
		return status;
	}

	return status;
}


ARC_STATUS demo()
{
	ARC_STATUS      status;
	sSYSTEM_INFO    sysInfo;		// see arclib.h for details			
	vector <int>    v1;				// devices index for creating raidset(SAS)

	memset(&sysInfo, 0, sizeof(sysInfo));

	// use the controller#0 for demonstrating
	status = ar->ArcGetSysInfo(&sysInfo);
	if ( status != ARC_SUCCESS )
	{
		printf("Failed to ArcGetSysInfo(...), Error Code = %X.\n", status);
		return status;
	}
	
	// check password
	//status = ar->ArcChkPassword("0000");
	if ( status != ARC_SUCCESS )
	{
		printf("Failed to ArcChkPassword(...), Error Code = %X.\n", status);
		return status;
	}

	if ( sysInfo.gsiTargetType == TARGET_SAS )
	{
		// print out the h/w monitor information for SAS controller
		status = Exe_SAS_Hw_Info(*ar);
	}
	else
	{
		// print out the h/w monitor information for Non-SAS controller
		status = Exe_Hw_Info(*ar);
	}

	if ( status != ARC_SUCCESS )
	{
		return status;
	}

	// print out the events of raid-subsystem
	status = Exe_Event_Info(*ar);
	if ( status != ARC_SUCCESS )
	{
		return status;
	}

	status = demo_create_raidset();

	return status;
}

ARC_STATUS DemoRAIDCtrl()
{
	int total_ctrl = 0;
	ARC_STATUS  status = ARC_FAILURE;	// see arclib.h for details
	sSYSTEM_INFO    sysInfo;			// see arclib.h for details	
	int retval = 1;

	HINSTANCE hInstDll;

#if (_MSC_VER >= 1600) // vs2010	
		hInstDll = LoadLibrary(ARCDLL_PATH);	
#else
	#error "Please use version of Visual Studio 2010 or later"
#endif

	if(!hInstDll) 
	{
		printf("Failed to load Areca DLL, please check the DLL path!!\n");
		return ARC_FAILURE;
	}

	pDllFindController = (DLLFINDCONTROLLER) GetProcAddress(hInstDll, "FindController");
	pDllGetController = (DLLGETCONTROLLER) GetProcAddress(hInstDll, "GetController");
	memset(&sysInfo, 0, sizeof(sysInfo));
	total_ctrl = pDllFindController();
	printf("%d controller(s) found.\n", total_ctrl);
	if ( total_ctrl )
	{				
		// get first controller
		ar = pDllGetController(0);
		if(ar)
		{
			// use the controller#0 for demonstrating
			status = ar->ArcGetSysInfo(&sysInfo);
			if ( status == ARC_SUCCESS )
			{
				status = demo();
				if ( status == ARC_SUCCESS )
				{
					retval = 0;
				}
			}
		}
	}

	FreeLibrary(hInstDll);

	return status;
}

int main(int argc, char* argv[])
{
	ARC_STATUS  status = ARC_FAILURE;		        

	/* demo for how to use RAID Controllers Library */
	status = DemoRAIDCtrl();

	printf("\n\n");

	/* IMPORTANT: DO NOT delete the instance of the "Interface" by yourself,
	   library will take care of that */

	return status;
}


